package rmrs.persist;

/*
 * Copyright (c) 1999 The Regenstrief Institute.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Written by Gunther Schadow.
 *
 * $Id: OID.java,v 1.1 1999/12/06 19:25:44 schadow Exp $
 */

/** An object identifier used with the persistence broker.
 *
 * @author Gunther Schadow
 * @version $Id: OID.java,v 1.1 1999/12/06 19:25:44 schadow Exp $ 
 */
public class OID 
    implements java.io.Serializable
{
    private long value;

    OID(long value) {
	this.value = value;
    }

    public String toString() {
	return Long.toString(value);
    }

    long toDatabaseValue() {
	return value;
    }
    
    static OID fromDatabaseValue(long dbvalue) {
	return new OID(dbvalue);
    }

    public int hashCode() {
	return (int)(value^(value>>>32));
    }

    /** Test for equality of two OIDs. */
    public boolean equals(Object o) {
	OID that = (OID)o;
	return that.value == this.value;
    }

    void setAsSQLParameter(java.sql.PreparedStatement stmt, int index)
	throws java.sql.SQLException
    {
	stmt.setLong(index, value);
    }

    static OID getFromSQLResult(java.sql.ResultSet rs, String column) 
	throws java.sql.SQLException
    {
	long value = rs.getLong(column);
	if(! rs.wasNull()) {
	    return new OID(value);
	} else {
	    return null;
	}
    }

    static OID getFromSQLResult(java.sql.ResultSet rs, int column) 
	throws java.sql.SQLException
    {
	long value = rs.getLong(column);
	if(! rs.wasNull()) {
	    return new OID(value);
	} else {
	    return null;
	}
    }
}
