package rmrs.persist;

/*
 * Copyright (c) 1999 The Regenstrief Institute.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Written by Gunther Schadow.
 *
 * $Id: ObjectSet.java,v 1.1 1999/12/06 19:25:44 schadow Exp $
 */

import java.util.Vector;
import java.lang.reflect.Field;
import java.util.Enumeration;
import java.sql.ResultSet;
import java.sql.SQLException;

/** A set collection of objects used for to-many associations and
 * returned by queries.
 *
 * @author Gunther Schadow
 * @version $Id: ObjectSet.java,v 1.1 1999/12/06 19:25:44 schadow Exp $ 
 */
public class ObjectSet 
{
    /** The cache contains eithe objects or OIDs */
    Vector cache;
    
    /** A JDBC ResultSet as returned by the query to browse
	through the results and selectively instantiate 
	Java objects. */
    ResultSet    jdbc_results;
    
    /** A persistence broker who issued the query */
    PersistenceBroker broker;

    /** The class that all objects in this set conform to. */
    Class theClass;

    /** object and field to close link back from demand-loaded objects */
    PersistentObject associationSource = null;
    Field            distalField = null;

    public ObjectSet(Class clazz) {
	theClass = clazz;
	cache = new Vector();
    }

    public ObjectSet(Class clazz, int size) {
	theClass = clazz;
	cache = new Vector(size);
    }

    void assignPersistenceBroker(PersistenceBroker pb) {
	broker = pb;
    }

    void registerLink(PersistenceBroker pb,
		      Field field, PersistentObject obj) {
	broker = pb;
	associationSource = obj;
	distalField = field;
    }

    PersistentObject get(int i)
	throws PersistException
    {
	if(cache != null) {
	    Object obj = cache.get(i);
	    if(obj != null) {
		try {
		    return (PersistentObject)obj;
		} catch(ClassCastException x) {
		    try {
			OID oid = (OID)obj;
			
			PersistentObject po = broker.load(theClass, oid);
			    
			if(! theClass.isInstance(po))
			    throw new 
				PersistException(PersistException
						 .CLASS_NOT_CONFORMANT, 
						 "oid " + oid + 
						 " resolved to a " + 
						 po.getClass().getName() + 
						 " in a set of " +
						 theClass.getName());

			if(associationSource != null &&
			   distalField != null) {

			    // set the backwards link in distal object 

			    try {
				distalField.set(po, associationSource);
			    } catch(IllegalAccessException z) {
				throw new 
				    PersistException(PersistException
						     .NO_ACCESS_TO_FIELD, 
						     "assigning back link", z);
			    }
			}

			cache.set(i, po);
			return po;
		    } catch(ClassCastException y) {
			// this is fatal because noone could have
			// added it ... shouldn't eve occur!
			throw new Error("FATAL ERROR illegal class " + 
					obj.getClass().getName() + 
					" in set of " +
					theClass.getName());
		    }
		}
	    }
	}
	
	return null;
    }

    public void add(PersistentObject obj) 
	throws PersistException
    {
	if(theClass.isInstance(obj)) {
	    cache.add(obj);
	} else {
	    throw new PersistException(PersistException.CLASS_NOT_CONFORMANT, 
				       "attempt to add a " + 
					obj.getClass().getName() + 
					" to a set of " +
					theClass.getName());
	}
    }
    
    void add(OID oid) {
	cache.add(oid);
    }

    public Enumeration elements() {
	return new Enumerator();
    }

    public class Enumerator 
	implements Enumeration
    {
	int i    = 0;
	int size = 0; 

	Enumerator() {
	    size = ObjectSet.this.cache.size();
	}

	public boolean hasMoreElements() {
	    return i < size;
	}

	public Object nextElement() {
	    if(i >= size)
		throw new java.util.
		    NoSuchElementException();
	    try {
		PersistentObject po = get(i++);
		return po;
	    } catch(PersistException x) {
		throw new Error(x.getMessage());
	    }
	}
    }
}
