package rmrs.persist;

/*
 * Copyright (c) 1999 The Regenstrief Institute.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Written by Gunther Schadow.
 *
 * $Id: RelClassMapping.java,v 1.1 1999/12/06 19:25:44 schadow Exp $
 */

import java.sql.Connection;
import java.sql.Statement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.lang.reflect.*;

/** Each one of these objects describe the mapping of one Java class
 * to a data base table.
 *
 * @author Gunther Schadow
 * @version $Id: RelClassMapping.java,v 1.1 1999/12/06 19:25:44 schadow Exp $ 
 */
class RelClassMapping {

    static final String CT_CLASS_NAME  = "jname";
    static final String CT_TABLE_NAME  = "name";
    static final String CT_MAKE_METHOD = "make";

    Class   theClass;
    String  theTable;
    Boolean useCtorToMake;
    Method  makeMethod = null;
    Vector  fieldMap = new Vector();

    RelClassMapping(ResultSet cs, Connection con) 
	throws PersistException, SQLException
    {
	// Get the java class
	try {
	    theClass = Class.forName(cs.getString(CT_CLASS_NAME.trim()));
	} catch(ClassNotFoundException x) { 
	    x.printStackTrace();
	}

	theTable = cs.getString(CT_TABLE_NAME).trim();

	// Which instantiation method to use: either the make method
	// field is NULL or it contains a name of a static method. If
	// it's NULL, objects are instantiated using the default
	// constructor <init>(). If it's a method name, the name
	// belongs to a static method taking no arguments and 
	// returning an instance of the class.

	String makeMethodName = cs.getString(CT_MAKE_METHOD);

	if(makeMethodName == null) {
	    // use constructor to create new instances

	    // check whether the class has a constructor with empty
	    // argument list, if not, we can't reconstruct the object
	    // from a file.

	    try {
		Constructor ctor = theClass.getDeclaredConstructor(null);

		// System.out.println(ctor + " " + ctor.isAccessible() 
		//                         + " " + ctor.getModifiers());
		// why has the accessibility no effect when it comes
		// to calling newInstance if the class has a package
		// level visible ctor?
		// System.out.println(ctor + " " + ctor.isAccessible());
		// ctor.setAccessible(true);
		// test: Object oi = theClass.newInstance();
		    
	    } catch(NoSuchMethodException x) {
		x.printStackTrace();
		throw new PersistException(PersistException.INVALID_CONSTRUCTOR,
					   "default constructor", x);
	    }

	} else {
	    // use alternative static method to create new instances

	    makeMethodName = makeMethodName.trim();
		
	    try {
		makeMethod = theClass.getMethod(makeMethodName, 
						null);
		    
		if(! Modifier.isStatic(makeMethod.getModifiers()))
		    throw new PersistException(PersistException.INVALID_CONSTRUCTOR, 
					       makeMethodName + 
					       " is not static");
		    
	    } catch(NoSuchMethodException x) {
		throw new PersistException(PersistException.INVALID_CONSTRUCTOR,
					   makeMethodName, x);
	    }		
	}

	// Get all the fields
	Statement stmt = con.createStatement();

	Field fields[] = theClass.getDeclaredFields();
	    
	for(int i = 0 ; i < fields.length; i++) {
	    Field f = fields[i];

	    if((f.getModifiers() & Modifier.STATIC) != 0)
		continue;

	    ResultSet rs = stmt
		.executeQuery("SELECT * FROM " + 
			      RelationalBroker.FIELD_TABLE + " WHERE " + 
			      RelFieldMapping.FT_CLASS_NAME + 
			      " = '" +  theClass.getName() + "' AND " +
			      RelFieldMapping.FT_FIELD_NAME + " = '" + 
			      f.getName() + "';");
		
	    if(rs.next()) {
		RelFieldMapping fieldMapping = 
		    new RelFieldMapping(f, rs);
		fieldMap.addElement(fieldMapping);
	    }
	    else if((f.getModifiers() & Modifier.TRANSIENT) == 0)
		System.err.println("warning: unmapped non-transient field "
				   + f.getName());
	}
    }
}

