package rmrs.persist;

/*
 * Copyright (c) 1999 The Regenstrief Institute.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Written by Gunther Schadow.
 *
 * $Id: RelFieldMapping.java,v 1.1 1999/12/06 19:25:44 schadow Exp $
 */

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Enumeration;
import java.lang.reflect.*;

/** Each one of these object describes the mapping of one Java field
 * to a data base field.
 *
 * @author Gunther Schadow
 * @version $Id: RelFieldMapping.java,v 1.1 1999/12/06 19:25:44 schadow Exp $ 
 */
class RelFieldMapping {
    
    static final String FT_CLASS_NAME  = "jcname";
    static final String FT_FIELD_NAME  = "jname";
    static final String FT_MODIFIER    = "modifier";
    static final String FT_COLUMN_NAME = "name";
    static final String FT_DISTAL_CLASS= "distal_jcname";
    static final String FT_DISTAL_FIELD= "distal_jname";

    static final int DIRECT       =             0x0001;
    static final int INDIRECT     =             0x0002;
    static final int INLINE       = INDIRECT  | 0x0004;
    static final int REFERENCE    = INDIRECT  | 0x0008;
    static final int REF_TO_ONE   = REFERENCE | 0x0010;
    static final int REF_TO_MANY  = REFERENCE | 0x0020;
    
    static final int CASCADE_LOAD   =           0x0100;
    static final int CASCADE_SAVE   =           0x0200;
    static final int CASCADE_DELETE =           0x0400;
    
    Field    theField;
    int      modifier;
    String   theColumn;
    Class    distalClass = null;
    Field    distalField = null;
    
    RelFieldMapping(Field f, ResultSet fs) 
	throws PersistException, SQLException
    {
	theField  = f;
	modifier  = fs.getInt(FT_MODIFIER);
	theColumn = fs.getString(FT_COLUMN_NAME);
	
	// persistent objects must not be inlined
	if(((modifier & INLINE) == INLINE) 
	   && theField.getType()
	   .isAssignableFrom(PersistentObject.class)) {
	    
	    throw new PersistException(PersistException.NO_INLINE,
				       theField.getType().getName());
	}
	
	// to-many links need the distal forreign key field
	if((modifier & REF_TO_MANY) == REF_TO_MANY)
	    {
		// Get the java class
		
		String distalClassName = 
		    fs.getString(FT_DISTAL_CLASS.trim());
		
		String distalFieldName = 
		    fs.getString(FT_DISTAL_FIELD.trim());		    
		
		try {
		    distalClass = 
			Class.forName(distalClassName);
		    
		    distalField = 
			distalClass.getDeclaredField(distalFieldName);
		    
		} catch(ClassNotFoundException x) { 
		    throw new PersistException(PersistException.CLASS_NOT_FOUND,
					       distalClassName, x);
		} catch(NoSuchFieldException x) { 
		    throw new PersistException(PersistException.FIELD_NOT_FOUND,
					       distalFieldName, x);
		}
		
		distalField.setAccessible(true);
	    }
	
	// this allows us to access the field later for save and
	// load operations ... isn't this a hack within Java?
	theField.setAccessible(true);
    }
}

