package rmrs.persist;

/*
 * Copyright (c) 1999 The Regenstrief Institute.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Written by Gunther Schadow.
 *
 * $Id: SQLSelect.java,v 1.1 1999/12/06 19:25:44 schadow Exp $
 */

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.ResultSet;
import java.util.Vector;
import java.util.Enumeration;

/** This is very far different from its siblings but has the same
 * purpose: allowing to build an SQL thing without having to mess
 * too much with SQL's intricacies.
 *
 * @author Gunther Schadow
 * @version $Id: SQLSelect.java,v 1.1 1999/12/06 19:25:44 schadow Exp $ 
 */
class SQLSelect
    extends SQLStatement
{
    public static final String op_eq = "=";
    public static final String op_lt = "<";
    public static final String op_gt = ">";
    public static final String op_le = "<=";
    public static final String op_ge = ">=";
    public static final String op_in = "IN";

    Vector tables   = new Vector();
    Vector fields   = new Vector();
    Vector criteria = new Vector();

    class Criterion {
	String field;
	String operator;
	Object value;
    
	Criterion(String field, String operator, Object value) {
	    this.field = field;
	    this.operator = operator;
	    this.value = value;
	}
    }
	  
    public SQLSelect() {
	super("SELECT ");
    }

    public void addTable(String name) {
	tables.add(name);
    }

    public void addField(String name) {
	fields.add(name);
    }

    public void addField(String field, Object value) {
	criteria.add(new Criterion(field, op_eq, value));
    }
    
    public void addCriterion(String name, String op, Object value) {
	criteria.add(new Criterion(name, op, value));
    }

    public void addCriterion(String criterion) {
	criteria.add(new Criterion(criterion, null, null));
    }

    // sub-select with operator being IN
    public void addCriterion(String name, SQLSelect subselect) {
	criteria.add(new Criterion(name, op_in, subselect));
    }

    protected void execute(Connection con) { }

    public ResultSet executeQuery(Connection con) 
	throws SQLException
    {
	buf = new StringBuffer("SELECT ");

	boolean first = true;
	Enumeration e = fields.elements();
	while(e.hasMoreElements()) {
	    String field = (String)e.nextElement();
	    
	    if(first)
		first = false;
	    else
		buf.append(", ");
	    
	    buf.append(field);
	}

	buf.append(" FROM ");

	first = true;
	e = tables.elements();
	while(e.hasMoreElements()) {
	    String table = (String)e.nextElement();
	    
	    if(first)
		first = false;
	    else
		buf.append(", ");
	    
	    buf.append(table);
	}

	buf.append(" WHERE ");

	first = true;
	e = criteria.elements();
	while(e.hasMoreElements()) {
	    Criterion crit = (Criterion)e.nextElement();
	    
	    if(first)
		first = false;
	    else
		buf.append(" AND ");
	    
	    buf.append(crit.field);

	    if(crit.operator != null) {
		buf.append(crit.operator);
		buf.append("?");
		args.add(crit.value);
	    }
	}
	
	buf.append(";");
	super.execute(con);
	return statement.executeQuery();
    }
}







