package rmrs.persist;

/*
 * Copyright (c) 1999 The Regenstrief Institute.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Written by Gunther Schadow.
 *
 * $Id: SQLStatement.java,v 1.1 1999/12/06 19:25:44 schadow Exp $
 */

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.util.Vector;
import java.util.Enumeration;
import java.sql.SQLException;

/** An abstract class to shield this ugly SQL thing from appearing
 * in too much of my source code. Almost to no avail!
 *
 * @author Gunther Schadow
 * @version $Id: SQLStatement.java,v 1.1 1999/12/06 19:25:44 schadow Exp $ 
 */
abstract class SQLStatement {
    
    protected StringBuffer buf;
    protected Vector       args = new Vector();
    protected PreparedStatement statement;

    abstract public void addField(String name, Object value);

    public void addKey(String name, Object value) {
	addField(name, value);
    }
    
    protected void execute(Connection con) 
	throws SQLException
    {
	statement = con.prepareStatement(buf.toString());
	Enumeration e = args.elements();
	int i = 1;
	while(e.hasMoreElements()) {
	    setParameter(statement, i, e.nextElement());
	    i++;
	}
	
	// MySQL always runs in auto-commit mode and no explicit commit
	// is allowed
	// FIXME: needs proper handling
	try {
	    con.setAutoCommit(false);
	} catch(SQLException x) {   }

	System.out.println(buf);
    }

    public void close() 
    {
	if(statement != null)
	    try { statement.close(); } catch(SQLException x) { }
    }

    protected SQLStatement(String init) {
	buf = new StringBuffer(init);
    }

    private void setParameter(PreparedStatement stmt, int index, Object obj) 
	throws SQLException
    {
	if(obj == null)
	    stmt.setNull(index, java.sql.Types.BIT);
	else {
	    Class cls = obj.getClass();
	    
	    // The following is sooooo darn stupid! Java 2 still holds
	    // on to these silly primitive types, makes everything so
	    // ugly. Besides I was hoping that parameter polymorphism
	    // would make type discrimiation in function names superflous,
	    // but these Java designers seem to think differently?
	    // I hate Java!
	    
	    if(cls == Boolean.class)
		stmt.setBoolean(index, ((Boolean)obj).booleanValue());
	    else if(cls == Byte.class)
		stmt.setByte(index, ((Byte)obj).byteValue());
	    else if(cls == Short.class)
		stmt.setShort(index, ((Short)obj).shortValue());
	    else if(cls == Integer.class)
		stmt.setInt(index, ((Integer)obj).intValue());
	    else if(cls == Long.class)
		stmt.setLong(index, ((Long)obj).longValue());
	    else if(cls == Float.class)
		stmt.setFloat(index, ((Float)obj).floatValue());
	    else if(cls == Double.class)
		stmt.setDouble(index, ((Double)obj).doubleValue());
	    else if(cls == java.math.BigDecimal.class)
		stmt.setBigDecimal(index, (java.math.BigDecimal)obj);
	    else if(cls == String.class)
		stmt.setString(index, (String)obj);
	    else if(cls == byte[].class)
		stmt.setBytes(index, (byte[])obj);
	    else if(cls == java.util.Date.class)
		stmt.setTimestamp(index, 
				  new java.sql.Timestamp(((java.util.Date)obj).getTime()));
	    else if(cls == java.sql.Timestamp.class)
		stmt.setTimestamp(index, (java.sql.Timestamp)obj);
	    else if(cls == java.sql.Date.class)
		stmt.setDate(index, (java.sql.Date)obj);
	    else if(cls == java.sql.Time.class)
		stmt.setTime(index, (java.sql.Time)obj);
	    else if(cls == OID.class)
		((OID)obj).setAsSQLParameter(stmt, index);
	    else
		throw new Error("unsupported class " + 
				obj.getClass().getName());
	}
    }
}
